/*******************************************************************************
 * Copyright (c) 2000, 2004 QNX Software Systems and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     QNX Software Systems - Initial API and implementation
 *******************************************************************************/

package idleLaunch;

import java.io.File;
import java.util.StringTokenizer;

import org.eclipse.cdt.core.IBinaryParser.IBinaryObject;
import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.debug.core.CDIDebugModel;
import org.eclipse.cdt.debug.core.ICDTLaunchConfigurationConstants;
import org.eclipse.cdt.debug.core.ICDebugConfiguration;
import org.eclipse.cdt.debug.core.cdi.CDIException;
import org.eclipse.cdt.debug.core.cdi.ICDISession;
import org.eclipse.cdt.debug.core.cdi.model.ICDIRuntimeOptions;
import org.eclipse.cdt.debug.core.cdi.model.ICDITarget;
import org.eclipse.cdt.debug.mi.core.MIException;
import org.eclipse.cdt.debug.mi.core.MISession;
import org.eclipse.cdt.debug.mi.core.cdi.model.Target;
import org.eclipse.cdt.debug.mi.core.command.CLICommand;
import org.eclipse.cdt.debug.mi.core.command.CommandFactory;
import org.eclipse.cdt.debug.mi.core.command.MIStackListFrames;
import org.eclipse.cdt.debug.mi.core.output.MIInfo;
import org.eclipse.cdt.launch.AbstractCLaunchDelegate;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.model.IProcess;

/**
 * @author User
 */
public class IdleLaunch extends AbstractCLaunchDelegate {

	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration, java.lang.String, org.eclipse.debug.core.ILaunch, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		monitor.beginTask("Idle mode", 10); //$NON-NLS-1$
		// check for cancellation
		if (monitor.isCanceled()) {
			return;
		}
		try {
			monitor.worked(1);
			IPath objPath = verifyProgramPath(configuration);
			ICProject project = verifyCProject(configuration);
			IBinaryObject objFile = verifyBinary(project, objPath);

			// set the default source locator if required
			setDefaultSourceLocator(launch, configuration);

			if (mode.equals(ILaunchManager.DEBUG_MODE)) {
				ICDebugConfiguration debugConfig = getDebugConfig(configuration);
				ICDISession dsession = null;
				String debugMode = configuration.getAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_START_MODE,
						ICDTLaunchConfigurationConstants.DEBUGGER_MODE_RUN);
				if (debugMode.equals(ICDTLaunchConfigurationConstants.DEBUGGER_MODE_RUN)) {

					// Create the session.
					dsession = debugConfig.createDebugger().createDebuggerSession(launch, objFile, new SubProgressMonitor(monitor, 8));
					monitor.worked(1);

					try {
						ICDITarget[] dtargets = dsession.getTargets();

						// Set the working directory, the environment and any arguments to the inferiors 
						try {
							String arguments[] = getProgramArgumentsArray(configuration);
							for (int i = 0; i < dtargets.length; ++i) {
								ICDIRuntimeOptions opt = dtargets[i].getRuntimeOptions();
								opt.setArguments(arguments);
								File wd = getWorkingDirectory(configuration);
								if (wd != null) {
									opt.setWorkingDirectory(wd.getAbsolutePath());
								}
								opt.setEnvironment(getEnvironmentAsProperty(configuration));
							}
						} catch (CDIException e) {
							abort("Failed setting runtime option though debugger", e, //$NON-NLS-1$
									ICDTLaunchConfigurationConstants.ERR_INTERNAL_ERROR);
						}
						monitor.worked(1);

						// Try to execute any extrac comand
						String[] commands = getExtraCommands(configuration);
						for (int i = 0; i < dtargets.length; ++i) {
							Target target = (Target)dtargets[i];
							MISession miSession = target.getMISession();
							for (int j = 0; j < commands.length; ++j ) {
								try {
									CLICommand cli = new CLICommand(commands[j]);
									miSession.postCommand(cli);
									MIInfo info = cli.getMIInfo();
									if (info == null) {
										throw new MIException("Timeout"); //$NON-NLS-1$
									}
								} catch (MIException e) {
									abort("Failed command", e, //$NON-NLS-1$
										ICDTLaunchConfigurationConstants.ERR_INTERNAL_ERROR);									
								}
							}
						}
						monitor.worked(2);

						// Try to detect if we have been attach/connected via "target remote localhost:port"
						// or "attach" and set the state to be suspended.
						for (int i = 0; i < dtargets.length; ++i) {
							Target target = (Target)dtargets[i];
							MISession miSession = target.getMISession();
							CommandFactory factory = miSession.getCommandFactory();
							try {
								MIStackListFrames frames = factory.createMIStackListFrames();
								miSession.postCommand(frames);
								MIInfo info = frames.getMIInfo();
								if (info == null) {
									throw new MIException("Timeout"); //$NON-NLS-1$
								}
								//@@@ We have to manually set the suspended state since we have some stackframes
								miSession.getMIInferior().setSuspended();
								miSession.getMIInferior().update();
							} catch (MIException e) {
								// If an exception is thrown that means ok
								// we did not attach/connect to any target.
							}
						}

						// create the Launch targets/processes for eclipse.
						boolean stopInMain = configuration.getAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN, false);
						for (int i = 0; i < dtargets.length; i++) {
							Target target = (Target)dtargets[i];
							target.setConfiguration(new IdleConfiguration(target));
							Process process = target.getProcess();
							IProcess iprocess = null;
							if (process != null) {
								iprocess = DebugPlugin.newProcess(launch, process, renderProcessLabel(objPath.toOSString()));
							}
							CDIDebugModel.newDebugTarget(launch, project.getProject(), dtargets[i], renderTargetLabel(debugConfig),
									iprocess, objFile, true, false, stopInMain, false);
						}

					} catch (CoreException e) {
						try {
							dsession.terminate();
						} catch (CDIException e1) {
							// ignore
						}
						throw e;
					}
				}
			} else {
				cancel("TargetConfiguration not supported", ICDTLaunchConfigurationConstants.ERR_INTERNAL_ERROR);
			}
		} finally {
			monitor.done();
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.cdt.launch.AbstractCLaunchDelegate#getPluginID()
	 */
	protected String getPluginID() {
		return IdleLaunchPlugin.PLUGIN_ID;
	}

	protected String[] getExtraCommands(ILaunchConfiguration configuration) throws CoreException {
		String commands = configuration.getAttribute(IIdleLaunchConfigurationConstants.ATTR_DEBUGGER_COMMANDS, new String());
		if (commands != null && commands.length() > 0) {
			StringTokenizer st = new StringTokenizer(commands, "\r\n");
			String[] cmds = new String[st.countTokens()];
			for (int i = 0; st.hasMoreTokens(); ++i) {
				cmds[i] = st.nextToken();
			}
			return cmds;
		}
		return new String[0];
	}
}
